# File tres/Scripts/btres.cl
# December 2, 2009
# By Doug Mink, SAO Telescope Data Center

# BTRES - Process multiple similar TRES bias images from raw data files

procedure btres (images)

string	images=""	{prompt="Images to process"}
bool	fixpreamp=no	{prompt="Run tpreamp to fix time constant (yes or no)"}
bool	fixpix=yes	{prompt="Fix bad pixels (yes or no)"}
bool	fixbias=no	{prompt="Fix bias section (yes or no)"}
bool	overscan=no	{prompt="Subtract overscan (yes or no)"}
bool	cosmic=yes	{prompt="Remove cosmic rays (yes or no)"}
real	threshold=10.0	{prompt="Cosmic ray threshold in sigma"}
char	sumspec="median" {prompt="Process combined 2-D file (mean median no)"}
bool	plothist=yes	{prompt="Plot histogram (yes or no)"}
bool	deltemp=yes	{prompt="Delete intermediate image files (yes or no)"}
bool	delimage=no	{prompt="Delete processed image file (yes or no)"}
bool	saveraw=no	{prompt="Save unprocessed image files (yes or no)"}
bool	saveold=no	{prompt="Save intermediate image files (yes or no)"}
bool	verbose=yes	{prompt="Print process progress (yes or no)"}

begin

file	imroot, imin, impath, combfile, biasfile
file	imout, ims, cdir, tempin, tempout
bool	vb, dt, sr, sv, disp, rc, delim, avg, ph
real	ct
string	sum, ddir, history

	vb = verbose
	dt = deltemp
	sv = saveold
	sr = saveraw
	ims = images
	delim = delimage
	ct = threshold
	sum = sumspec
	ph = plothist

	if (substr (ims,1,1) == "@")
	    fileroot (substr (ims,2,100))
	else
	    fileroot (ims)
	trssum.outroot = fileroot.rootname

	if (ims == "") {
	    print ("BTRES: Reduce multiple similar TRES dark or bias images, removing cosmic rays")
	    print ("Usage: btres images with both amps to be corrected")
	    return
	    }

# If input image is a list prepend @ if not already there
	smatch (ims, ".list")
	if (smatch.found) {
	    if (!access (ims)) {
		print ("BTRES: ",ims," list file not found")
		return
		}
	    if (substr (ims,1,1) != '@')
		ims = "@" // images
	    }

	tcheck
	if (tcheck.rawdir)
	    trsdate (tcheck.datedir)

#  Make list file from input file parameter
	trsfiles.instdir = ctres.instdir
	trsfiles.datedir = ctres.datedir
	ddir = ctres.datedir
	trsfiles.copyfile = yes
	trsfiles.overwrite = no
	trsfiles.adddate = no
	trsfiles.keeppath = no
	trsfiles.extension = ".fits"
	trsfiles (ims)
	if (trsfiles.nfiles == 0) {
	    delete (trsfiles.tempin)
	    if (vb)
		print ("BTRES: No files found: ", ims)
	    return
	    }
	tempin = trsfiles.tempin
	tempout = trsfiles.tempout
	ims = "@" // tempout

# Remove overscan, trim, and merge amplifiers of image
# Then remove pixel to pixel variation from image (including IR fringing)
# Then extract spectra from image and add dispersion function
	tproc.gaincorr = 1.0
	tproc.fixbias = fixbias
	tproc.fixpix = fixpix
	tproc.fixpreamp = fixpreamp
	tproc.overscan = overscan
	tproc.subbias = no
	tproc.subdark = no
	tproc.flatten = no
	tproc.cosmic = cosmic
	tproc.threshold = ct
	tproc.sumspec = sumspec
	tproc.deltemp = dt
	tproc.saveraw = sr
	tproc.saveold = sv
	tproc.verbose = vb
	tproc.rmscat = no
	tcosmic.savebp = sv
	tproc (ims, flatten=no, extract=no, disperse=no)

	if (delim) {
	    imdelete (ims)
	    }

# Copy combined bias file to standard name
	combfile = tproc.sumfile
	if (combfile != "") {
	    tfib1 (combfile, specdir="")
	    biasfile = "bias" // tfib1.binning // ".fits"
	    if (access (biasfile))
		delete (biasfile)
	    copy (combfile, biasfile)
	    gettime (0.0, timediff=99)
	    history = ddir // "/" // combfile
	    hedit (biasfile, "BIASFILE", history, add+, verify-)
	    if (ph)
		thist (biasfile,intbin+,logy-,listout-)
	    }

#  Delete temporary list files
	if (dt) {
	    delete (tempin)
	    delete (tempout)
	    }


	if (tcheck.rawdir)
	    tdata (ddir)
end

# Feb 13 2009	New task based on FTRES
# Mar  9 2009	Return if no files found from list
# Apr 10 2009	Make default normdir = "" (working directory)
# Apr 27 2009	Read raw and working directories from CTRES
# May 12 2009	Drop avgspec parameter; add mean and median options to sumspec
# May 12 2009	Add cosmic parameter to make cosmic ray removal optional
# Jun 17 2009	Add delimage parameter to delete all image files but dark*fits
# Jun 19 2009	Make overscan subtraction optional
# Jun 19 2009	Write standard named file bias[binning].fits
# Jun 22 2009	Drop unused parameters for flattening and dark subtraction
# Jul  2 2009	Set all parameters of TRSFILES
# Sep  2 2009	Turn off scattered light removal; it is not needed
# Sep 30 2009	Exit with error message if list file is not found
# Nov  9 2009	Set saveraw default to no
# Dec  2 2009	Add option to plot histogram
