# File tres/Scripts/dtres.cl
# December 2, 2009
# By Doug Mink, SAO Telescope Data Center

# DTRES - Process multiple similar TRES dark images from raw data files

procedure dtres (images)

string	images=""	{prompt="Images to process"}
bool	fixpreamp=no	{prompt="Run tpreamp to fix time constant (yes or no)"}
bool	fixpix=yes	{prompt="Fix bad pixels (yes or no)"}
bool	fixbias=no	{prompt="Fix bias section (yes or no)"}
bool    overscan=yes	{prompt="Subtract overscan (yes or no)"}
bool	subbias=no	{prompt="Subtract bias frame (yes or no)"}
real	gaincorr=1.0	{prompt="Gain correction for amp2 to amp1"}
bool	cosmic=yes	{prompt="Remove cosmic rays (yes or no)"}
real	threshold=10.0	{prompt="Cosmic ray threshold in sigma"}
string	sumspec="median" {prompt="Compute a mean or median dark image (mean median no)"}
bool	plothist=yes	{prompt="Plot histogram (yes or no)"}
bool	logy=yes	{prompt="Plot log of number in histogram (yes or no)"}
bool	deltemp=yes	{prompt="Delete intermediate image files (yes or no)"}
bool	delimage=no	{prompt="Delete processed image file (yes or no)"}
bool	saveraw=no	{prompt="Save unprocessed image files (yes or no)"}
bool	saveold=no	{prompt="Save intermediate image files (yes or no)"}
bool	verbose=yes	{prompt="Print process progress (yes or no)"}

begin

file	imin, combfile, darkfile
file	ims, tempin, tempout
string	history, cmb, ddir
bool	vb, dt, sr, sv, delim, ph, ly
real	ct

	vb = verbose
	dt = deltemp
	sv = saveold
	sr = saveraw
	ims = images
	delim = delimage
	ct = threshold
	ph = plothist
	ly = logy

	if (ims == "") {
	    print ("DTRES: Reduce multiple similar TRES dark images, removing cosmic rays")
	    print ("Usage: dtres images with both amps to be corrected")
	    return
	    }

# If input image is a list prepend @ if not already there
	smatch (ims, ".list")
	if (smatch.found) {
	    if (!access (ims)) {
		print ("DTRES: ",ims," list file not found")
		return
		}
	    if (substr (ims,1,1) != '@')
		ims = "@" // ims
	    }
	if (vb) {
	    if (substr (ims,1,1) == "@")
		print ("DTRES: Processing dark files from ", ims)
	    else
		print ("DTRES: Processing dark files ", ims)
	    }

	tcheck
	if (tcheck.rawdir)
	    trsdate (tcheck.datedir)

#  Make list file from input file parameter
	trsfiles.instdir = ctres.instdir
	trsfiles.datedir = ctres.datedir
	ddir = ctres.datedir
	trsfiles.copyfile = yes
	trsfiles.overwrite = no
	trsfiles.adddate = no
	trsfiles.keeppath = no
	trsfiles.extension = ".fits"
	trsfiles (ims)
	if (trsfiles.nfiles == 0) {
	    delete (trsfiles.tempin)
	    if (vb)
		print ("DTRES: No files found: ", ims)
	    return
	    }
	tempin = trsfiles.tempin
	tempout = trsfiles.tempout
	ims = "@" // tempout

# Remove overscan, trim, and merge amplifiers of image
	tproc.gaincorr = gaincorr
	tproc.fixbias = fixbias
	tproc.fixpix = fixpix
	tproc.fixpreamp = fixpreamp
	tproc.overscan = overscan
	tproc.subbias = subbias
	tproc.subdark = no
	tproc.darkfile = ""
	tproc.flatten = no
	tproc.cosmic = cosmic
	tproc.threshold = ct
	tproc.sumspec = sumspec
	tproc.rmscat = no
	tproc.deltemp = dt
	tproc.saveraw = sr
	tproc.saveold = sv
	tproc.verbose = vb
	tcosmic.savebp = sv
	tproc (ims, extract=no, disperse=no)

	combfile = tproc.sumfile
	if (combfile != "") {
	    tfib1 (combfile)
	    darkfile = "dark" // tfib1.binning // ".fits"
	    if (access (darkfile))
		delete (darkfile)
	    copy (combfile, darkfile)
	    gettime (0.0, timediff=99)
	    history = ddir // "/" // combfile
	    hedit (darkfile, "DARKFILE", history, add+, verify-)
	    if (ph)
		thist (darkfile, logy=ly, intbin+, listout-)
	    }

	if (delim) {
	    imdelete (ims)
	    }

#  Delete temporary list files
	if (dt) {
	    delete (tempin)
	    delete (tempout)
	    }

	if (tcheck.rawdir)
	    tdata (ddir)
end

# Apr 28 2009	New task based on BTRES
# May 12 2009	Use TPROC sumfile instead of parameter from combining subroutine
# May 12 2009	Add cosmic parameter to make cosmic ray removal optional
# May 14 2009	Change default sumspec to median
# May 18 2009	Set tproc.subdark =no; add apsize to output name
# May 22 2009	Drop apsize from output name
# Jun 22 2009	Add subbias and overscan to choose background removal
# Jun 22 2009	Drop unused parameters for flattening
# Jul  2 2009	Set all parameters of TRSFILES
# Sep  2 2009	Turn off scattered light removal; it doesn't work for darks
# Sep 30 2009	Exit with error message if list file is not found
# Nov  9 2009	Set saveraw default to no
# Dec  2 2009	Add option to plot histogram of result
