# tres/Scripts/tharshift1.cl
# December 10, 2009
# By Doug Mink

# THARSHIFT1 - Find the correct pixel shift from ThAr spectra
#	       for one object spectrum

procedure tharshift1 (spectrum)

string	spectrum=""	{prompt="Spectrum for which to find a pixel shift"}
file    refspec1=""     {prompt="Closest ThAr spectrum before
(returned)"}
real    refwt1=0.0      {prompt="Weight for refspec1 (returned)"}
file    refspec2=""     {prompt="Closest ThAr spectrum after
(returned)"}
real    refwt2=0.0      {prompt="Weight for refspec2 (returned)"}

real	shift=0.0	{prompt="Pixel shift (returned)"}
string	refpath=""	{prompt="Reference ID ThAr file path (returned)"}
string	refroot="comp" {prompt="Root for reference ThAr spectrum"}
string	refselect="interp" {prompt="Reference selection method (closest
interp reference two)", enum="closest|interp|reference|two"}
bool    verbose=yes	{prompt="Print process progress (yes or no)"}
bool    debug=no	{prompt="Print debugging info (yes or no)"}

struct *thartab

begin

    string apfib, smjd, sshift, rsel, rroot, s1, s2, s3, s4, tstr, rpath
    file tharfile, spthar, spec
    bool vb, ow, db, dt, first
    real mjd, mjd0, shift0, mjd1, mjd2, shift1, shift2, diff1, diff2

    rsel = refselect
    vb = verbose
    db = debug
    spec = spectrum
    rroot = refroot

# Find aperture/binning/fiber size code to check ThAr spectra
    tfib1 (spec, specdir="", debug=db)
    mjd0 = tfib1.mjdmid
    apfib = tfib1.apfib
    if (db)
	print ("THARSHIFT1: ",spec," ",mjd0," apfib=",apfib)

# Set up database file name for this configuration
    if (mode == "reference") {
	shift = 0.0
	return
	}

# Set up database file name for this configuration
    tharfile = rroot // apfib // ".shift"
    if (!access (tharfile)) {
	thartable (apfib=apfib,deltemp=yes,verbose=vb,debug=db)
	if (thartable.outfile == "") {
	    shift = 0.0
	    refpath = ""
	    return
	    }
	if (!access (tharfile)) {
	    print ("THARSHIFT1: Cannot create ",tharfile)
	    return
	    }
	}
    if (debug)
	print ("THARSHIFT1: Reading ",tharfile, " refmode = ", rsel)

    first = yes
    mjd1 = mjd0
    mjd2 = mjd0
    diff1 = 0.0
    diff2 = 0.0
    refwt1 = 0.0
    refwt2 = 0.0
    shift = 0.0

# Read two header lines
    thartab = tharfile
    if (fscan (thartab, s1,s2,s3,s4) == EOF) {
	if (db)
	    print ("THARSHIFT1: cannot read line 1 of ",tharfile)
	}
    if (fscan (thartab, s1,s2,s3,s4) == EOF) {
	if (db)
	    print ("THARSHIFT1: cannot read line 2 of ",tharfile)
	}

# Read ThAr file name, MJD, and shift from each line
    while (fscan (thartab, spthar, rpath, smjd, sshift) != EOF) {
	if (db) {
	    print ("THARSHIFT1: ",spthar," ", rpath, " ",smjd, " ", sshift)
	    }
	mjd = real (smjd)
	if (diff1 == 0.0) {
	    mjd1 = mjd
	    if (mjd < mjd0)
		diff1 = mjd0 - mjd
	    else
		diff1 = mjd - mjd0
	    shift1 = real (sshift)
	    refspec1 = spthar
	    mjd2 = mjd
	    if (mjd < mjd0)
		diff2 = mjd0 - mjd
	    else
		diff2 = mjd - mjd0
	    shift2 = real (sshift)
	    refspec2 = spthar
	    refpath = rpath
	    }
	else if (mjd < mjd0 && (mjd0 - mjd) < diff1) {
	    mjd1 = mjd
	    diff1 = mjd0 - mjd
	    shift1 = real (sshift)
	    refspec1 = spthar
	    refpath = rpath
	    }
	else if (mjd > mjd0 && (mjd - mjd0) < diff2) {
	    mjd2 = mjd
	    diff2 = mjd - mjd0
	    shift2 = real (sshift)
	    refspec2 = spthar
	    refpath = rpath
	    }
	}

# Mode interp: Interpolate linearly between two closest ThAr spectra
    if (mjd2 > mjd0 && mjd1 < mjd0) {
	if (rsel == "interp") {
	    shift = shift1 + ((mjd0 - mjd1) / (mjd2 - mjd1)) * (shift2 - shift1)
	    if (debug)
		printf ("THARSHIFT1: shift= %.5f (%.5f,%.5f) MJD= %.5f (%.5f,%.5f)\n",
	    }
			shift,shift1,shift2,mjd0,mjd1,mjd2)
	else if (rsel == "two") {
	    refwt1 = diff2 / (diff1 + diff2)
	    refwt2 = diff1 / (diff1 + diff2)
	    tstr = refspec1 // " " // refwt1
#	    hedit (spectrum, "REFSPEC1", tstr, add+, update+,verify-,show=db)
	    tstr = refspec2 // " " // refwt2
#	    hedit (spectrum, "REFSPEC2", tstr, add+, update+,verify-,show=db)
	    if (debug) {
		printf ("THARSHIFT1: spec1 = %s, weight= %.3f\n", refspec1, refwt1)
		printf ("THARSHIFT1: spec2 = %s, weight= %.3f\n", refspec2, refwt2)
		}
	    }
	}
    else
	rsel = "closest"

# Mode closest: Use shift from closest ThAr spectrum
    if (rsel == "closest") {
	if (mjd2 > mjd0 && mjd1 < mjd0) {
	    if (diff1 < diff2) {
		shift = shift1
		if (debug)
		    printf ("THARSHIFT1: shift= %.5f at %.5f from %.f5\n",shift,mjd,mjd1)
#		hedit (spectrum, "REFSPEC1", refspec1, add+, update+,verify-,show=vb)
		}
	    else if (diff2 < diff1) {
		shift = shift2
		if (debug)
		    printf ("THARSHIFT1: shift= %.5f at %.5f from %.f5\n",shift,mjd,mjd2)
#		hedit (spectrum, "REFSPEC1", refspec2, add+, update+,verify-,show=vb)
		}
	    else if (shift2 != shift1) {
		shift = shift1 + ((mjd0 - mjd1) / (mjd2 - mjd1)) * (shift2 - shift1)
		if (debug)
		    printf ("THARSHIFT1: shift= %.5f (%.5f,%.5f) MJD= %.5f (%.5f,%.5f)\n",
			    shift,shift0,shift1,mjd0,mjd1,mjd2)
#		hedit (spectrum, "REFSPEC1", refspec1, add+, update+,verify-,show=vb)
		}
	    else if (mjd2 > mjd0) {
		shift = shift2
		if (debug)
		    printf ("THARSHIFT1: shift= %.5f at %.5f from %.f5\n",shift,mjd0,mjd2)
#		hedit (spectrum, "REFSPEC1", refspec2, add+, update+,verify-,show=vb)
		}
	    else if (mjd1 > mjd0) {
		shift = shift1
		if (debug)
		    printf ("THARSHIFT1: shift= %.5f at %.5f from %.f5\n",shift,mjd0,mjd1)
#		hedit (spectrum, "REFSPEC1", refspec1, add+, update+,verify-,show=vb)
		}
	    else {
		shift = 0.0
		if (debug)
			printf ("THARSHIFT1: shift= 0.0 at %.5f\n",shift,mjd0)
		}
	    }
	}
    else if (rsel != "interp" && rsel != "two") {
	print ("THARSHIFT1: Mode ",rsel, " not implemented")
	shift = 0.0
	}
    if (vb) {
	printf ("%s pixel shift is %.5f at %.5f (%s) from %s\n", spec, shift, mjd0, rsel, refpath)
	}
    thartab = ""
    return
end

# May 23 2008	New task
# May 30 2008	Use trsfib to get center MJD of exposure
# Jun  6 2008	Call TRSFIB1 instead of TRSFIB
# Jul 18 2008	Add modes for shift interpolation
# Jul 21 2008	Add refroot parameter
# Aug 12 2008	Set REFSPEC1 (and REFSPEC2 with weights if refselect=interp)

# Dec 10 2009	Return refspec1, refspec2, refwt1, and refwt2 as parameters
