# File tres/Scripts/tsfiles.cl
# By Doug Mink
# October 6, 2009

# TSFILES -- Make list file of images given a range of sequence numbers
#            or a list of file names

procedure tfiles (images)

string images="1-300"	{prompt="Files to process"}

bool adddate=no		{prompt="Prepend observation date to file name (yes or no)"}
bool keeppath=no	{prompt="Return full pathname in list, not just filename (yes or no)"}
string extension=".fits" {prompt="Extension of files to be listed"}
file tempin=""		{prompt="List file made from images (returned)"}
file tempout=""		{prompt="List file of real image names (returned)"}
int maxlfile=0		{prompt="Maximum length of file names (returned)"}
int maxlobj=0		{prompt="Maximum length of object names (returned)"}
int nfiles=0		{prompt="Number of files in lists (returned)"}
bool copyfile=no	{prompt="Copy files to current directory (yes or no)"}
bool overwrite=no	{prompt="Overwrite existing files (yes or no)"}
bool verbose=yes	{prompt="Print table of image parameters (yes or no)"}
bool debug=no		{prompt="Print table with more image parameters (yes or no)"}

begin
    bool vb, db, cp, ad, ow, isfits, kp
    file idir, imdir
    file impath, imin, imfile
    string ims, ddir, ext
    int n, nrange, lfile, lobj
    string specs, olist, lastpath, obj, tempcode

    vb = verbose
    db = debug
    cp = copyfile
    ow = overwrite
    ad = adddate
    kp = keeppath
    ims = images
    ext = extension
    if (ext == ".fits")
	isfits = yes
    else
	isfits = no
    nfiles = 0
    imdir = trsdate.wdir

#  Make temporary list of input files
    tempcode = mktemp ("")
    tempin = "tempin" // tempcode // ".list"
    if (access (tempin))
	delete (tempin)
    isrange (ims)
    if (isrange.range) {
	getrange (ims, 0)
	nrange = getrange.nval
	for (i = 1; i <= nrange; i = i + 1) {
	    getrange (ims, i)
	    if (i == 1)
		print (getrange.ival, > tempin)
	    else
		print (getrange.ival, >> tempin)
	    }
	}
    else
	files (ims, >tempin)
    tempout = "tempout" // tempcode // ".list"
    if (access (tempout))
	delete (tempout)
    maxlfile = 0
    maxlobj = 0

# Make list of image paths if instrument directory is specified
    if (imdir != "" || isrange.range) {
	fixdir (imdir)
	imdir = fixdir.newdir
	if (!access (imdir)) {
	    print ("TSFILES cannot read directory ",imdir)
	    return
	    }
	lastpath = ""

	list = tempin
	while (fscan (list,imin) != EOF) {

	    if (substr (imin, 1, 1) == "#")
		next

# If a number is given, prepend appropriate zeroes and wildcard
	    isnum (imin)
	    if (isnum.fpcode > 0) {
		n = int (imin)
		if (n < 10)
		    imin = "000" // n // ".*" // ext
		else if (n < 100)
		    imin = "00" // n // ".*" // ext
		else if (n < 1000)
		    imin = "0" // n // ".*" // ext
		else
		    imin = n // ".*" // ext
		}

# If file extension is missing, add wildcard
	    else {
		smatch (imin, ".fits")
		if (smatch.found == no)
		    imin = imin // "*" // ext
		}
	    impath = imdir // imin
	    filepath.files = ""
	    filepath (impath)
	    impath = filepath.files
	    if (impath == "") {
		next
		}
	    smatch (impath, ",")
	    if (smatch.found) {
		impath = substr (impath,1,smatch.start-1)
		}
	    if (!access (impath)) {
		print ("TSFILES: Cannot read file ",impath)
		next
		}
	    tres.filename (impath)
	    imfile = tres.filename.filename
	    imdir = tres.filename.dirpath
	    lfile = strlen (imfile)
	    if (lfile > maxlfile)
		maxlfile = lfile
	    if (impath != lastpath) {
		if (cp) {
		    if (ad) {
			imfile = ddir // "." // imfile
			}
		    if (ow) {
			if (!access (imfile)) {
			    copy (impath, imfile)
			    if (db)
				print ("TSFILES: Copying file ",impath," to ",imfile)
			    }
			}
		    else {
			if (access (imfile))
			    delete (imfile)
			copy (impath, imfile)
			if (db)
			    print ("TSFILES: Copying file ",impath," to ",imfile)
			}
		    if (isfits) {
			    tfib1 (impath)
			if (tfib1.imagetype == "ERROR") {
			    if (db)
				print ("TSFILES: Not reading file ",impath)
			    }
			else if (kp) {
			    print (impath, >>tempout)
			    if (vb && !db)
				print (impath)
			    }
			else {
			    print (imfile, >>tempout)
			    if (vb && !db)
				print (imfile)
			    }
			}
		    else if (kp) {
			print (impath, >>tempout)
			if (vb && !db)
			    print (impath)
			}
		    else {
			print (imfile, >>tempout)
			if (vb && !db)
			    print (imfile)
			}
		    }
		else if (isfits) {
		    tfib1 (impath)
		    if (tfib1.imagetype == "ERROR") {
			if (db)
			    print ("TSFILES: Not reading file ",impath)
			}
		    else {
			print (impath, >>tempout)
			if (db)
			    print ("TSFILES: Reading file ",impath)
			else if (vb)
			    print (impath)
			}
		    }
		else {
		    print (impath, >>tempout)
		    if (db)
			print ("TSFILES: Reading file ",impath)
		    else if (vb)
			print (impath)
		    }
		}
	    lastpath = impath
	    lobj = strlen (tfib1.object)
	    if (lobj > maxlobj)
		maxlobj = lobj
	    nfiles = nfiles + 1
	    }
	}
    else
	tempout = tempin
    return
end

# Oct  5 2009	New task based on TRSFILES
# Oct  6 2009	End file pathname just before first comma, if one is present
