# File tres/Scripts/twoid.cl
# By Doug Mink
# December 10, 2009

# TWOID -- Combine line positions from two ecidentify database files

procedure twoid (dbin1, dbwt1, dbin2, dbwt2, dbout)

string dbin1=""		{prompt="First input file to process"}
real dbwt1=0.0		{prompt="Weight for firstd input file"}
string dbin2=""		{prompt="Second input file to process"}
real dbwt2=0.0		{prompt="Weight for second input file"}
string dbout=""		{prompt="File for output"}

string indir=""		{prompt="Input directory with database directory"}
string outdir=""	{prompt="Output directory with database directory"}
bool keepfit=no		{prompt="Keep dispersion fit (yes or no)"}
bool verbose=yes	{prompt="Log progress (yes or no)"}
bool debug=no		{prompt="Print output (yes or no)"}

struct *ecdb1
struct *ecdb2

begin

    bool vb, db, kf, first
    int i, io, ncoeff, icoeff
    file idbpath1, idbpath2, idbdir, odbpath, odbdir
    string idbfile, odbfile, idbroot, odbroot
    string s1, s2, s3, s4, s5, s6, s7, s8
    string t1, t2, t3, t4, t5, t6, t7, t8
    real x, w1, w2, shiftmin, lwt1, lwt2

    vb = verbose
    db = debug
    kf = keepfit

# Input database directory
    if (indir != "") {
	fixdir (indir)
	idbdir = fixdir.newdir // "database/"
	}
    else {
	idbdir = "database/"
	}

# First input dispersion fit file
    idbfile = dbin1
    fileroot (idbfile)
    idbroot = fileroot.rootname
    idbpath1 = idbdir // "ec" // idbroot

# Second input dispersion fit file
    if (dbin2 != "") {
	idbfile = dbin2
	fileroot (idbfile)
	idbroot = fileroot.rootname
	idbpath2 = idbdir // "ec" // idbroot
	}
    else
	idbpath2 = ""

# Output dispersion fit file
    if (outdir != "") {
	fixdir (outdir)
	odbdir = fixdir.newdir // "database/"
	}
    else {
	odbdir = "database/"
	}
    odbfile = dbout
    fileroot (odbfile)
    odbroot = fileroot.rootname
    odbpath = odbdir // "ec" // odbroot

# Make output directory if it does not already exist
    if (!access (odbdir))
	mkdir (obdir)

# Quit if input database entry is not there
    if (!access (idbpath1)) {
	print ("TWOID: Database file ",idbpath1, " does not exist")
	return
	}
    if (!access (idbpath2)) {
	print ("TWOID: Database file ",idbpath2, " does not exist")
	return
	}

# If output database entry already exists, delete it
    if (access (odbpath))
	delete (odbpath)

# If files have same name and no shift, just copy the original
    if (dbin1 == dbout) {
	copy (idbpath1, odbpath)
	if (vb) {
	    print ("TWOID: Copying ",idbpath1," to ", odbpath)
	    }
	return
	}

    if (vb) {
	print ("TWOID: Mixing ",idbpath1," and ",idbpath2," to ",odbpath
	}
    s1 = ""
    s2 = ""
    s3 = ""
    s4 = ""
    s5 = ""
    s6 = ""
    s7 = ""
    s8 = ""
    t1 = ""
    t2 = ""
    t3 = ""
    t4 = ""
    t5 = ""
    t6 = ""
    t7 = ""
    t8 = ""
    w1 = 0.0
    w1a = 0.0
    ncoeff = 0
    gettime (0.0, timediff=99, usesec+)

# Read through database entry one line at a time
    first = no
    ecdb1 = idbpath1
    ecdb2 = idbpath2
    while (fscan (ecdb1, s1, s2, s3, s4, s5, s6, s7, s8) != EOF) {
	if (s4 != "")
	    w1 = real (s4)
	else
	    w1 = 0.0

# Read next line of second file if first file wavelength is greater
	if (w1 == 0.0 || (w1 > 0.0 && w1a < w1))
	    fscan (ecdb2, t1, t2, t3, t4, t5, t6, t7, t8) != EOF) {

# Blank line
	if (s1 == "") {
	    printf ("\n", >>odbpath)
	    if (db) {
		printf ("\n")
		}
	    }

# Date and time if comment line
	else if (s1 == "#") {
	    if (first)
		printf ("# %s %s %s\n", gettime.dweek, gettime.date, gettime.time, >odbpath)
	    else
		printf ("# %s %s %s\n", gettime.dweek, gettime.date, gettime.time, >>odbpath)
	    first = no
	    if (db) {
		printf ("# %s %s %s\n", gettime.dweek, gettime.date, gettime.time)
		}
	    }

#	else if (ncoeff > 0 && s2 == "") {
#	    printf (\t\t%s\n", s1)
#	    }

# Two-token lines are parameter assignments
	else if (s3 == "") {
	    if (s1 == "id") {
		printf ("\tid\t%s\n", odbroot, >>odbpath)
		if (db) {
		    printf ("\tid\t%s\n", odbroot)
		    }
		}
	    else if (s1 == "image") {
		printf ("\timage\t%s\n", odbroot, >>odbpath)
		if (db) {
		    printf ("\timage\t%s\n", odbroot)
		    }
		}
	    else if (s1 == "slope") {
		printf ("\tslope\t%s\n", s2, >>odbpath)
		if (db) {
		    printf ("\tslope\t%s\n", s2)
		    }
		if (!keepfit) {
		    break
		    }
		}
	    else if (s1 == "coefficents") {
		icoeff=0
		ncoeff = int (s2)
		}
	    else {
		printf ("\t%s\t%s\n", s1, s2, >>odbpath)
		if (db) {
		    printf ("\t%s\t%s\n", s1, s2)
		    }
		}
	    }

# Three-token lines
	else if (s4 == "") {
	    printf ("begin\tecidentify %s\n", odbroot, >>odbpath)
	    if (db) {
		printf ("begin\tecidentify %s\n", odbroot)
		}
	    }

# Calibration emission lines: s3 is line center pixel coordinate, s1 is order
	else {
	    w1 = real (s4)
	    w1a = real (t4)
	    while (w1 < w1a) {
		if (fscan (ecdb1, s1, s2, s3, s4, s5, s6, s7, s8) != EOF)
		    w1 = real (s4)
		else
		    break
		}
	    while (w1 > w1a) {
		if (fscan (ecdb2, t1, t2, t3, t4, t5, t6, t7, t8) != EOF) {
		    w1a = real (t4)
		else
		    break
		}
	    if (w1 == w1a) {
		x = (real (s3) * lwt1) + (real (t3) + lwt2)
		w2 = real (s5)
		i = int (s1)
		io = int (s2)
		printf ("\t\t%2d %4d %10.4f %11.5f  %10.4f   %s  %s  %s\n", i, io, x, w1, w2, s6, s7, s8, >>odbpath)
		if (db) {
		    printf ("\t\t%s %s %s %s %s %s %s %s\n", s1, s2, s3, s4, s5, s6, s7, s8)
		    printf ("\t\t%s %s %s %s %s %s %s %s\n", t1, t2, t3, t4, t5, t6, t7, t8)
		    printf ("\t\t%2d %4d %10.4f %11.5f  %10.4f   %s  %s  %s\n", i, io, x, w1, w2, s6, s7, s8)
		    }
		}
	    }
	s1 = ""
	s2 = ""
	s3 = ""
	s4 = ""
	s5 = ""
	s6 = ""
	s7 = ""
	s8 = ""
	}
end

# Dec 10 2009	New program based on TMODID
