# File t_timfill.x
# May 21, 2009
# By Doug Mink

include <error.h>
include <ctype.h>
include <imhdr.h>
include <imio.h>
include <mach.h>

define	MAXCOL		4096
define	MAXLINE		4700

# TIMFILL -- Fill zeroed pixels with values interpolated across lines

procedure t_timfill ()

pointer	im1, im2, linein, lineout
real	dataout[MAXCOL,MAXLINE]
char	infile[SZ_FNAME]
char	outfile[SZ_FNAME]
#char	datedir[SZ_FNAME]
char	dstr[SZ_LINE]
char	hstr[SZ_LINE]
long	v[IM_MAXDIM]
bool	verbose, debug
int	i, j, ix, nc, nl, nfill, iz, nz, pz[MAXCOL]
real	datai, data1, rindef, ddata, dnz, dz
pointer	imgl2r(), immap()
int	impnlr()
bool	clgetb()
long	clktime()

begin

	rindef = INDEFR

# Get parameters
	call clgstr ("infile", infile, SZ_FNAME)
	call clgstr ("outfile", outfile, SZ_FNAME)
	verbose = clgetb ("verbose")
	debug = clgetb ("debug")

# Open the input image
	iferr (im1 = immap (infile, READ_ONLY, 0)) {
	    call printf ("TIMFILL: Cannot read %s\n")
		call pargstr (infile)
	    call clputi ("error", 1)
	    return
	    }
	nc = IM_LEN(im1,1)
	nl = IM_LEN(im1,2)
	if (verbose) {
	    call printf ("TIMFILL: %d x %d input flattening image %s opened\n")
		call pargi (nc)
		call pargi (nl)
		call pargstr (infile)
	    call flush (STDOUT)
	    }

# Read from the input image and process each line
	nfill = 0
	do j = 1, nl {

# Get input buffer for this line of the image
	    linein = imgl2r (im1, j)
	    if (linein == NULL) {
		call printf ("TIMFILL: Cannot read line %d of %s\n")
		    call pargi (i)
		    call pargstr (infile)
		call flush (STDOUT)
		call clputi ("error", 3)
		call imunmap (im1)
		return
		}

# Step across a row, filling in pixels
	    nz = 0
	    dnz = 0.0
	    data1 = rindef
	    do i = 1, nc {
		datai = Memr[linein+i-1]

# Keep column number if pixel is zero
		if (datai == 0.0) {
		    nz = nz + 1
		    dnz = dnz + 1.0
		    pz[nz] = i
		    }

# If first nonzero pixel after zeroed pixels
		else if (nz > 0) {

# If there was a nonzero pixel before zeroed ones, interpolate
		    if (data1 != rindef) {
			ddata = (datai - data1) / dnz
			do iz = 1, nz {
			    ix = pz[iz]
			    dz = float (iz)
			    dataout[ix,j] = data1 + (dz * ddata)
			    nfill = nfill + 1
			    if (debug) {
				call printf ("TIMFILL: (%d,%d) = %.3f (%.3f+%.3f)\n")
				    call pargi (pz[iz])
				    call pargi (j)
				    call pargr (dataout[ix,j])
				    call pargr (data1)
				    call pargr (dz*ddata)
				}
			    }
			dataout[i,j] = datai
			nz = 0
			dnz = 0.0
			}

# Otherwise set previous zeroed pixels to value of this one
		    else {
			do iz = 1, nz {
			    ix = pz[iz]
			    dataout[ix,j] = datai
			    nfill = nfill + 1
			    if (debug) {
				call printf ("TIMFILL: (%d,%d) = %.3f (%.3f+%.3f)\n")
				    call pargi (pz[iz])
				    call pargi (j)
				    call pargr (dataout[ix,j])
				    call pargr (data1)
				    call pargr (dz*ddata)
				}
			    }
			dataout[i,j] = datai
			nz = 0
			dnz = 0.0
			}
		    data1 = datai
		    }

# If previous pixel not zeroed, keep this one
		else {
		    dataout[i,j] = datai
		    data1 = datai
		    }
		}

# If last pixels are zeroes, set them to last non-zero value
	    if (nz > 0 && data1 != rindef) {
		ddata = (datai - data1) / dnz
		do iz = 1, nz {
		    dz = float (iz)
		    ix = pz[iz]
		    dataout[ix,j] = data1 + (dz * ddata)
		    nfill = nfill + 1
		    }
		dataout[i,j] = datai
		nz = 0
		dnz = 0.0
		}
#	    if (debug) {
#		call printf ("TIMFILL: Line %4d set.\n")
#		    call pargi (j)
#		call flush (STDOUT)
#		}
	    }

	if (verbose) {
	    call printf ("TIMFILL: %d x %d image filled\n")
		call pargi (nc)
		call pargi (nl)
	    call flush (STDOUT)
	    }

# Open the output mask file
	iferr (im2 = immap (outfile, NEW_COPY, im1)) {
	    call printf ("TIMFILL: Cannot create output file %s\n")
		call pargstr (outfile)
	    call flush (STDOUT)
	    call clputi ("error", 2)
	    return
	    }
	else if (debug) {
	    call printf ("TIMFILL: output file %s created\n")
		call pargstr (outfile)
	    call flush (STDOUT)
	    }

	call imastr (im2, "i_title", "TRES background")
	nc = IM_LEN(im2,1)
	nl = IM_LEN(im2,2)
	if (verbose) {
	    call printf ("TIMFILL: %d x %d output image %s opened\n")
		call pargi (nc)
		call pargi (nl)
		call pargstr (outfile)
	    call flush (STDOUT)
	    }

# Set FILLIMG header keyword
	call logtime (dstr, SZ_LINE)
#	if (verbose) {
#	    call printf ("%s Background file filled from %s/%s\n")
#		call pargstr (dstr)
#		call pargstr (datedir)
#		call pargstr (infile)
#	    call flush (STDOUT)
#	    }
	call sprintf (hstr,SZ_LINE,"%s %d pixels interpolated")
	    call pargstr (infile)
	    call pargi (nfill)
	call imastr (im2, "FILLIMG", hstr)
	if (verbose) {
	    call printf ("FILLIMG: %s\n")
		call pargstr (hstr)
	    call flush (STDOUT)
	    }
	IM_CTIME(im2) = clktime (long(0))
	IM_UPDATE(im2) = YES

# Initialize vector for output
	call amovkl (long(1), v, IM_MAXDIM)

# Write the output file one line at a time
	do j = 1, nl {

# Get output buffer for this line of the image
	    v[2] = j
	    if (impnlr (im2, lineout, v) < nc) {
		call printf ("TIMFILL: Cannot write line %d of %s\n")
		    call pargi (j)
		    call pargstr (outfile)
		    call flush (STDOUT)
		call clputi ("error", 3)
		return
		}
	    call achtrr (dataout[1,j], Memr[lineout], nc)
	    }
	if (verbose) {
	    call printf ("TIMFILL: Filled image transferred to output buffer\n")
	    call flush (STDOUT)
	    }

# Close output image
	if (verbose) {
	    call printf ("TIMFILL: Ready to write filled image %s\n")
		call pargstr (outfile)
	    call flush (STDOUT)
	    }
	call imunmap (im2)
	if (verbose) {
	    call printf ("TIMFILL: Output filled image %s written.\n")
		call pargstr (outfile)
	    call flush (STDOUT)
	    }

# Close input image
	call imunmap (im1)
	if (verbose) {
	    call printf ("TIMFILL: Input image %s closed.\n")
		call pargstr (infile)
	    call flush (STDOUT)
	    }

end
# Apr 22 2009	New program based on tres.tmaskfile
# May 21 2009	Drop unused variables
